/*
 * Click nbfs://nbhost/SystemFileSystem/Templates/Licenses/license-default.txt to change this license
 */
package com.jcg.profilingexample;

import java.io.File;
import java.io.IOException;
import org.netbeans.lib.profiler.heap.HeapSummary;
import org.netbeans.lib.profiler.heap.Heap;
import org.netbeans.lib.profiler.heap.HeapFactory;
import java.util.List;
import org.netbeans.lib.profiler.heap.Instance;
import org.netbeans.lib.profiler.heap.JavaClass;

/**
 *
 * @author omozegieaziegbe
 */
public class ProfilingExample {

    private static Heap heap;

    private static void analyzeHeapDump() throws IOException {
        heap = HeapFactory.createHeap(new File("heapdump.hprof"));

        HeapSummary summary = heap.getSummary();

        System.out.println("Total number of instances: " + summary.getTotalLiveInstances());
        System.out.println("Memory usage in bytes: " + summary.getTotalLiveBytes());
        System.out.println("GC Roots: " + heap.getGCRoots().size());
        System.out.println("Total number of Classes: " + heap.getAllClasses().size());
        System.out.println("Timestamp: " + summary.getTime());

    }

    public static void librarySummary() {
        JavaClass libraryClass = heap.getJavaClassByName("com.jcg.profilingexample.Library");
        if (libraryClass == null) {
            System.out.println("Library class not located");
            return;
        }
        List<Instance> instances = libraryClass.getInstances();
        long totalSize = 0;
        long instancesNumber = libraryClass.getInstancesCount();
        for (Instance instance : instances) {
            totalSize += instance.getSize();
        }
        System.out.println("Number of Library instances: " + instancesNumber);
        System.out.println("Memory consumed by Library instances: " + totalSize);
    }

    public static void main(String[] args) throws IOException {
        Library cityLibrary = new Library(1, "Programming Library");
        cityLibrary.addBook("Java Programming Essentials");
        cityLibrary.addBook("Data Structures and Algorithms");
        cityLibrary.addSection("Technology");
        cityLibrary.addSection("Literature");

        Library homeLibrary = new Library(1, "Philosophical Library");
        homeLibrary.addBook("Age of Reason");
        homeLibrary.addBook("The Principles of Nature");
        homeLibrary.addSection("Philosophy");

        File f = new File("heapdump.hprof");
        if (f.exists()) {
            f.delete();
        }

        HeapDumpUtility.dumpHeap("heapdump.hprof", true);

        analyzeHeapDump();
        
        librarySummary();
    }
}
